using System.Collections;
using System.Windows.Forms;

using System;

namespace CocoAddin.AtgParser {



public class Parser {
	const int _EOF = 0;
	const int _ident = 1;
	const int _number = 2;
	const int _string = 3;
	const int _badString = 4;
	const int _char = 5;
	const int maxT = 39;

	const bool T = true;
	const bool x = false;
	const int minErrDist = 2;
	
	public Scanner scanner;
	public Errors  errors;

	public Token t;    // last recognized token
	public Token la;   // lookahead token
	int errDist = minErrDist;

const int id = 0;
	const int str = 1;
	
	//static bool genScanner;
	//static string tokenString;         // used in declarations of literal tokens
	//static string noString = "-none-"; // used in declarations of literal tokens
	
	static TreeNode root;			   // the root of the TreeNode wie build during parsing
	
	public static TreeNode getRoot(){
		return root;
	}

/*-------------------------------------------------------------------------*/



	public Parser(Scanner scanner) {
		this.scanner = scanner;
		errors = new Errors();
	}

	void SynErr (int n) {
		if (errDist >= minErrDist) errors.SynErr(la.line, la.col, n);
		errDist = 0;
	}

	public void SemErr (string msg) {
		if (errDist >= minErrDist) errors.Error(t.line, t.col, msg);
		errDist = 0;
	}
	
	void Get () {
		for (;;) {
			t = la;
			la = scanner.Scan();
			if (la.kind <= maxT) { ++errDist; break; }
				if (la.kind == 40) {
				}

			la = t;
		}
	}
	
	void Expect (int n) {
		if (la.kind==n) Get(); else { SynErr(n); }
	}
	
	bool StartOf (int s) {
		return set[s, la.kind];
	}
	
	void ExpectWeak (int n, int follow) {
		if (la.kind == n) Get();
		else {
			SynErr(n);
			while (!StartOf(follow)) Get();
		}
	}
	
	bool WeakSeparator (int n, int syFol, int repFol) {
		bool[] s = new bool[maxT+1];
		if (la.kind == n) { Get(); return true; }
		else if (StartOf(repFol)) return false;
		else {
			for (int i=0; i <= maxT; i++) {
				s[i] = set[syFol, i] || set[repFol, i] || set[0, i];
			}
			SynErr(n);
			while (!s[la.kind]) Get();
			return StartOf(syFol);
		}
	}
	
	void atg() {
		while (StartOf(1)) {
			Get();
		}
		Expect(6);
		Expect(1);
		root = new TreeNode("COMPILER "+t.val); 
		root.ImageIndex = 0;
		root.Tag = t;
		
		while (StartOf(2)) {
			Get();
		}
		ScannerSpecification(root);
		ParserSpecification(root);
		Expect(7);
		Expect(1);
		Expect(8);
	}

	void ScannerSpecification(TreeNode n) {
		TreeNode s = new TreeNode("Scanner");
		s.ImageIndex = 1;
		s.Tag = t;
		n.Nodes.Add(s);
		
		if (la.kind == 9) {
			Get();
			TreeNode c = new TreeNode("Characters");
			c.ImageIndex = 4;
			c.Tag = t;
			s.Nodes.Add(c);
			
			while (la.kind == 1) {
				SetDecl(c);
			}
		}
		if (la.kind == 10) {
			Get();
			TreeNode tn = new TreeNode("Tokens");
			tn.ImageIndex = 3;
			tn.Tag = t;
			s.Nodes.Add(tn);
			
			
			while (la.kind == 1 || la.kind == 3 || la.kind == 5) {
				TokenDecl(tn);
			}
		}
		if (la.kind == 11) {
			Get();
			TreeNode p = new TreeNode("Pragmas");
			p.ImageIndex = 5;
			p.Tag = t;
			s.Nodes.Add(p);
			
			
			while (la.kind == 1 || la.kind == 3 || la.kind == 5) {
				PragmaDecl(p);
			}
		}
		while (la.kind == 25) {
			CommentDecl(s);
		}
		while (la.kind == 29) {
			IgnoreDecl(s);
		}
	}

	void ParserSpecification(TreeNode n) {
		TreeNode ps = new TreeNode("Parser");
		ps.ImageIndex = 7;
		ps.Tag = t;
		n.Nodes.Add(ps);
		
		Expect(31);
		TreeNode pr = new TreeNode("Productions");
		pr.ImageIndex = 5;
		pr.Tag = t;
		ps.Nodes.Add(pr);
		
		while (la.kind == 1) {
			Production(pr);
		}
	}

	void SetDecl(TreeNode sd) {
		Expect(1);
		TreeNode tn = new TreeNode(t.val);
		tn.ImageIndex = 8;
		tn.Tag = t;
		sd.Nodes.Add(tn);
		
		Expect(12);
		Set();
		Expect(8);
	}

	void TokenDecl(TreeNode tn) {
		Symbol();
		TreeNode e = new TreeNode(t.val);
		e.ImageIndex = 10;
		e.Tag = t;
		tn.Nodes.Add(e);
		
		if (la.kind == 12) {
			Get();
			TokenExpr();
			Expect(8);
		}
	}

	void PragmaDecl(TreeNode p) {
		TokenDecl(p);
		if (la.kind == 36) {
			SemAction();
		}
	}

	void CommentDecl(TreeNode c) {
		Expect(25);
		TreeNode n = new TreeNode("Comments");
		n.ImageIndex = 6;
		n.Tag = t;
		c.Nodes.Add(n);
		
								
		Expect(26);
		TokenExpr();
		string from = t.val; 
		string nested = " ..... ";
		
		Expect(27);
		TokenExpr();
		string to = t.val; 
		if (la.kind == 28) {
			Get();
			nested = " NESTED "; 
		}
		TreeNode m = new TreeNode(from + nested + to);
		m.ImageIndex = 9;
		m.Tag = t;
		n.Nodes.Add(m);
		
	}

	void IgnoreDecl(TreeNode i) {
		Expect(29);
		TreeNode tn = new TreeNode("Ignore");
		tn.ImageIndex = 11;
		tn.Tag = t;
		i.Nodes.Add(tn);
		
		if (StartOf(3)) {
			Set();
			TreeNode s = new TreeNode(t.val);
			s.ImageIndex = 3;
			s.Tag = t;
			tn.Nodes.Add(s);
			
		} else if (la.kind == 30) {
			Get();
		} else SynErr(40);
	}

	void Set() {
		BasicSet();
		while (la.kind == 13 || la.kind == 14) {
			if (la.kind == 13) {
				Get();
			} else {
				Get();
			}
			BasicSet();
		}
	}

	void BasicSet() {
		if (la.kind == 3) {
			Get();
		} else if (la.kind == 1) {
			Get();
		} else if (la.kind == 5) {
			Get();
			if (la.kind == 15) {
				Get();
				Expect(5);
			}
		} else if (la.kind == 16) {
			Get();
		} else SynErr(41);
	}

	void Symbol() {
		if (la.kind == 1) {
			Get();
		} else if (la.kind == 3) {
			Get();
		} else if (la.kind == 5) {
			Get();
		} else SynErr(42);
	}

	void TokenExpr() {
		TokenTerm();
		while (la.kind == 17) {
			Get();
			TokenTerm();
		}
	}

	void TokenTerm() {
		TokenFactor();
		while (StartOf(4)) {
			TokenFactor();
		}
		if (la.kind == 18) {
			Get();
			Expect(19);
			TokenExpr();
			Expect(20);
		}
	}

	void TokenFactor() {
		if (la.kind == 1 || la.kind == 3 || la.kind == 5) {
			Symbol();
		} else if (la.kind == 19) {
			Get();
			TokenExpr();
			Expect(20);
		} else if (la.kind == 21) {
			Get();
			TokenExpr();
			Expect(22);
		} else if (la.kind == 23) {
			Get();
			TokenExpr();
			Expect(24);
		} else SynErr(43);
	}

	void SemAction() {
		Expect(36);
		while (StartOf(5)) {
			Get();
		}
		Expect(37);
	}

	void Production(TreeNode p) {
		Expect(1);
		TreeNode i = new TreeNode(t.val);
		i.ImageIndex = 7;
		i.Tag = t;
		p.Nodes.Add(i);
		
		if (la.kind == 34) {
			Attributes();
		}
		if (la.kind == 36) {
			SemAction();
		}
		Expect(12);
		Expression();
		Expect(8);
	}

	void Attributes() {
		Expect(34);
		while (StartOf(6)) {
			Get();
		}
		Expect(35);
	}

	void Expression() {
		Term();
		while (la.kind == 17) {
			Get();
			Term();
		}
	}

	void Term() {
		if (la.kind == 38) {
			Resolver();
		}
		Factor();
		while (StartOf(7)) {
			Factor();
		}
	}

	void Resolver() {
		Expect(38);
		Expect(19);
		while (StartOf(8)) {
			Get();
		}
		Expect(20);
	}

	void Factor() {
		switch (la.kind) {
		case 1: case 3: case 5: case 32: {
			if (la.kind == 32) {
				Get();
			}
			Symbol();
			if (la.kind == 34) {
				Attributes();
			}
			break;
		}
		case 19: {
			Get();
			Expression();
			Expect(20);
			break;
		}
		case 21: {
			Get();
			Expression();
			Expect(22);
			break;
		}
		case 23: {
			Get();
			Expression();
			Expect(24);
			break;
		}
		case 16: {
			Get();
			break;
		}
		case 33: {
			Get();
			break;
		}
		case 36: {
			SemAction();
			break;
		}
		default: SynErr(44); break;
		}
	}



	public void Parse() {
		la = new Token();
		la.val = "";		
		Get();
		atg();

    Expect(0);
	}
	
	bool[,] set = {
		{T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x},
		{x,T,T,T, T,T,x,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, x},
		{x,T,T,T, T,T,T,T, T,x,x,x, T,T,T,T, T,T,T,T, T,T,T,T, T,x,T,T, T,x,T,x, T,T,T,T, T,T,T,T, x},
		{x,T,x,T, x,T,x,x, x,x,x,x, x,x,x,x, T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x},
		{x,T,x,T, x,T,x,x, x,x,x,x, x,x,x,x, x,x,x,T, x,T,x,T, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x},
		{x,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,x,T,T, x},
		{x,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,x, T,T,T,T, x},
		{x,T,x,T, x,T,x,x, x,x,x,x, x,x,x,x, T,x,x,T, x,T,x,T, x,x,x,x, x,x,x,x, T,T,x,x, T,x,x,x, x},
		{x,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, x,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, T,T,T,T, x}

	};
} // end Parser


public class Errors {
	public int count = 0;                                    // number of errors detected
  public string errMsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text
	
	public void SynErr (int line, int col, int n) {
		string s;
		switch (n) {
			case 0: s = "EOF expected"; break;
			case 1: s = "ident expected"; break;
			case 2: s = "number expected"; break;
			case 3: s = "string expected"; break;
			case 4: s = "badString expected"; break;
			case 5: s = "char expected"; break;
			case 6: s = "\"COMPILER\" expected"; break;
			case 7: s = "\"END\" expected"; break;
			case 8: s = "\".\" expected"; break;
			case 9: s = "\"CHARACTERS\" expected"; break;
			case 10: s = "\"TOKENS\" expected"; break;
			case 11: s = "\"PRAGMAS\" expected"; break;
			case 12: s = "\"=\" expected"; break;
			case 13: s = "\"+\" expected"; break;
			case 14: s = "\"-\" expected"; break;
			case 15: s = "\"..\" expected"; break;
			case 16: s = "\"ANY\" expected"; break;
			case 17: s = "\"|\" expected"; break;
			case 18: s = "\"CONTEXT\" expected"; break;
			case 19: s = "\"(\" expected"; break;
			case 20: s = "\")\" expected"; break;
			case 21: s = "\"[\" expected"; break;
			case 22: s = "\"]\" expected"; break;
			case 23: s = "\"{\" expected"; break;
			case 24: s = "\"}\" expected"; break;
			case 25: s = "\"COMMENTS\" expected"; break;
			case 26: s = "\"FROM\" expected"; break;
			case 27: s = "\"TO\" expected"; break;
			case 28: s = "\"NESTED\" expected"; break;
			case 29: s = "\"IGNORE\" expected"; break;
			case 30: s = "\"CASE\" expected"; break;
			case 31: s = "\"PRODUCTIONS\" expected"; break;
			case 32: s = "\"WEAK\" expected"; break;
			case 33: s = "\"SYNC\" expected"; break;
			case 34: s = "\"<\" expected"; break;
			case 35: s = "\">\" expected"; break;
			case 36: s = "\"(.\" expected"; break;
			case 37: s = "\".)\" expected"; break;
			case 38: s = "\"IF\" expected"; break;
			case 39: s = "??? expected"; break;
			case 40: s = "invalid IgnoreDecl"; break;
			case 41: s = "invalid BasicSet"; break;
			case 42: s = "invalid Symbol"; break;
			case 43: s = "invalid TokenFactor"; break;
			case 44: s = "invalid Factor"; break;

			default: s = "error " + n; break;
		}
		Console.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void SemErr (int line, int col, int n) {
		Console.WriteLine(errMsgFormat, line, col, ("error " + n));
		count++;
	}

	public void Error (int line, int col, string s) {
		Console.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void Exception (string s) {
		Console.WriteLine(s); 
		System.Environment.Exit(1);
	}
} // Errors

}
