// <file>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Stefan Wallner" />
// </file>

 using System;
 using System.Text;
 using System.IO;
 using System.Collections;
 using System.Diagnostics;
 using System.Drawing;
 using System.Reflection;
 using System.Windows.Forms;
 using System.CodeDom.Compiler;
 using System.Collections.Generic;

 using ICSharpCode.Core;		
 using ICSharpCode.SharpDevelop;
 using ICSharpCode.SharpDevelop.Project;
 using ICSharpCode.SharpDevelop.Gui;
 using ICSharpCode.SharpDevelop.DefaultEditor.Gui.Editor;
 using ICSharpCode.TextEditor;
 using ICSharpCode.TextEditor.Document;
 
 using CocoAddin.Tools;
 using CocoAddin.AtgParser;
 using CocoAddin.Commands;

namespace CocoAddin
{
	/// <summary>
	/// Pad for the CocoAddin
	/// </summary>
	public class CocoPad : AbstractPadContent
	{
		private static CocoPad instance = null;
		
		Panel panel;
		TreeView treeView;
		
		//ControlMenue on the Pad
		ToolStrip toolStrip;
		
		//the active parsed file wich is shown in the coco treeview
 		private string activeParsedFile = ""; 
			
 		//initialize the gui for the CocoAddin
		public CocoPad()
		{
			LoggingService.Debug("Starte CocoPad !");
			if (instance != null){
				throw new CocoAddinException("CocoPad: Just one instance of CocoPad allowed !");
			}
			
			instance = this;
			panel = new Panel();
			
			toolStrip = ToolbarService.CreateToolStrip(this, "/SharpDevelop/Pads/CocoAddinPad/Toolbar");
			toolStrip.GripStyle = ToolStripGripStyle.Hidden;
			toolStrip.Dock = DockStyle.Top;
				
 			//ImageList
 			Assembly assembly = Assembly.GetExecutingAssembly();
 			
 			ImageList imageList = new ImageList();
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.compiler"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.scanner"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.parser"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.token_head"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.character_head"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.production_head"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.comment_head"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.production"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.character"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.comment"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.token"));
 			imageList.Images.Add(ResourceService.GetIcon("CocoAddin.ignore_head"));
 			
 			//TreeView
 			treeView = new TreeView();
 			treeView.ShowPlusMinus = true;
 			treeView.Dock = DockStyle.Fill;
 			
 			//assign the ImageList to the TreeView
 			treeView.BeginUpdate();
 			treeView.ImageList = imageList;
 			treeView.AfterSelect += new TreeViewEventHandler(SelectTreeView);
 			treeView.EndUpdate();	
 			
 			panel.Controls.Add(treeView);
 			panel.Controls.Add(toolStrip);
 			
			//listeners 
			WorkbenchSingleton.Workbench.ActiveWorkbenchWindowChanged += new EventHandler(WorkbenchWindowChanged);
 			WorkbenchSingleton.Workbench.ViewOpened += new ViewContentEventHandler(WorkbenchWindowOpened);
		}
		
		/// <summary>
		/// set the active parsed file in the instance
		/// </summary>
		static public string ActiveParsedFile {
			get {
					
				if (instance == null){ return ""; }
				else{ return instance.activeParsedFile; }
				}
			
			set {
				if (instance != null){
					instance.activeParsedFile = value;
					instance.RedrawContent();
				} 
			}
		}
	
		/// <summary>
		/// updates treeview
		/// </summary>
 		void UpdateTreeView(bool save){
				
 			IWorkbenchWindow w;
 			try {	
 				
	 			w = WorkbenchSingleton.Workbench.ActiveWorkbenchWindow;	
	 			if (w != null && w.ViewContent.FileName.ToLower().EndsWith(".atg")) {
 					
 					//used because Workbench.ActiveWorkbenchWindowChanged is called two times from SharDevelop 
 					if (!save && w.ViewContent.FileName.Equals(ActiveParsedFile)) {
 						return;
 					}
 						
 					//update the toolbar with the new parsed filename
					this.activeParsedFile = w.ViewContent.FileName;
 					ToolbarService.UpdateToolbar(toolStrip);
					ToolbarService.UpdateToolbarText(toolStrip);
					
	 				Parser p = new Parser(new Scanner(w.ViewContent.FileName));
	 				p.Parse();
	 				TreeNode root = Parser.getRoot();
	 				treeView.Update();
	 				treeView.Nodes.Clear();
	 				treeView.Nodes.Add(root);
	 				treeView.EndUpdate();
	 				treeView.ExpandAll(); 
	 				treeView.BringToFront();
	 			}
 			} catch(Exception) {
 				LoggingService.Debug("No Tree to build !");
 			}
 		}
		
		/// <summary>
		/// The <see cref="System.Windows.Forms.Control"/> representing the pad
		/// </summary>
		public override Control Control {
			get {
				return panel;
			}
		}
				
		/// <summary>
		/// Cleans up all used resources
		/// </summary>
		public override void Dispose()
		{
			instance = null;
			panel.Dispose();
			
		}
		
		/// <summary>
		/// node in the treeview selected
		/// </summary>
 		void SelectTreeView(object sender, TreeViewEventArgs e) {
 			try { 				 				 			
	 			TextEditorControl textArea = null;
	 			IWorkbenchWindow window = WorkbenchSingleton.Workbench.ActiveWorkbenchWindow;
	 				 				 			
	 			if (window != null && window.ViewContent.FileName == this.activeParsedFile){
	 				textArea = ((ITextEditorControlProvider)window.ViewContent).TextEditorControl;
	 			} else {
	 				FileService.OpenFile(this.activeParsedFile);
	 				textArea = ((ITextEditorControlProvider)window.ViewContent).TextEditorControl;
	 			}
	 			Token tk = (Token)e.Node.Tag;
	 			
	 			//jump to the selected area and mark the token
	 			textArea.ActiveTextAreaControl.JumpTo(tk.line-1, tk.col);
	 			textArea.ActiveTextAreaControl.TextArea.SelectionManager.ClearSelection();
	 			textArea.ActiveTextAreaControl.TextArea.SelectionManager.SetSelection(new Point(tk.col-1,tk.line-1),new Point(tk.col + e.Node.Text.Length - 1,tk.line-1));
	 			textArea.ActiveTextAreaControl.Refresh();
	 			
 			} catch(Exception ex){
 				LoggingService.Error("Error in SelectTreeView " + ex);
 			} 		 			
 		}
			
		/// <summary>
 		/// updates the TreeView
 		/// </summary>
		void WorkbenchWindowChanged(object sender, EventArgs e){
 			UpdateTreeView(false);
 		}	
 		
 		/// <summary>
 		/// a new file is opened
 		/// </summary>
 		void WorkbenchWindowOpened(object sender, ViewContentEventArgs e){ 
 			e.Content.Saved += new SaveEventHandler(FileSaved);
 			UpdateTreeView(false);
 			WorkbenchSingleton.Workbench.GetPad(typeof(CocoPad)).BringPadToFront();
 		}
 			
 		/// <summary>
 		/// actual atg file is saved
 		/// </summary>
 		void FileSaved(object sender, SaveEventArgs e){
 			UpdateTreeView(true);
 		}
	}
}
