// <file>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Stefan Wallner" />
// </file>


using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Windows.Forms;

using ICSharpCode.Core; 
using ICSharpCode.SharpDevelop.Gui;
using ICSharpCode.SharpDevelop.Internal.ExternalTool;
using ICSharpCode.SharpDevelop.Gui.XmlForms;

using CocoAddin.Tools;

namespace CocoAddin.Gui
{
	/// <summary>
	/// the Option Panel for the configuration of Coco/R
	/// </summary>
	public class CocoParameterOption : AbstractOptionPanel
	{
		CocoProperties cocoOptions = new CocoProperties();
		
		/// <summary>
		/// load the contents of the panel from the xfrm-File
		/// </summary>
		public override void LoadPanelContents() {
			
			SetupFromXmlStream(this.GetType().Assembly.GetManifestResourceStream("CocoAddin.Resources.CocoOptions.xfrm"));		
			
			//load the actual state into the optionpanel
			try {
				((TextBox)ControlDictionary["packageBox"]).Text = cocoOptions.CocoPackage;
				((TextBox)ControlDictionary["traceBox"]).Text = cocoOptions.CocoTrace;
				((TextBox)ControlDictionary["frameBox"]).Text = cocoOptions.CocoFrame;
				((TextBox)ControlDictionary["optionBox"]).Text = cocoOptions.OptionExtended;
				((PictureBox)ControlDictionary["sswlogo"]).Image = (Image)ResourceService.GetBitmap("CocoAddin.sswlogo");
				
				//loading the ListView for coco location
				ListView l = (ListView)ControlDictionary["CocoListView"];
				string[] locations = cocoOptions.CocoExe.Split(';');
				
				l.BeginUpdate();
				foreach(string s in locations){
					if (s != "") l.Items.Add(new ListViewItem(s));
				}
				int choosed = System.Convert.ToInt32(PropertyService.Get("Addins.CocoAddin.CocoExe.Selected","0"));
				if (l.Items.Count > 0)
					l.Items[choosed].Selected = true;
				l.EndUpdate();
				
			} catch(Exception ex){
				LoggingService.Error("Could not load all iptions !" + ex);
			}
			
			((Button)ControlDictionary["addButton"]).Click += new EventHandler(SelectCocoClicked);
			((Button)ControlDictionary["changeButton"]).Click += new EventHandler(ChangeButtonClicked);
			((Button)ControlDictionary["removeButton"]).Click += new EventHandler(RemoveButtonClicked);
			((Button)ControlDictionary["frameButton"]).Click += new EventHandler(FrameButtonClicked);
			
			if (((ListView)ControlDictionary["CocoListView"]).Items.Count == 0){
				((Button)ControlDictionary["changeButton"]).Enabled = false;
				((Button)ControlDictionary["removeButton"]).Enabled = false;
			}
		}
		
		/// <summary>
		/// store changes made from user
		/// </summary>
		public override bool StorePanelContents(){
			//store coco commandline
			cocoOptions.CocoPackage = ((TextBox)ControlDictionary["packageBox"]).Text;
			cocoOptions.CocoTrace = ((TextBox)ControlDictionary["traceBox"]).Text;
			cocoOptions.CocoFrame = ((TextBox)ControlDictionary["frameBox"]).Text;
			cocoOptions.OptionExtended = ((TextBox)ControlDictionary["optionBox"]).Text;
			
			//store coco executables
			ArrayList tmp = new ArrayList();
			foreach(ListViewItem item in ((ListView)ControlDictionary["CocoListView"]).Items){
				tmp.Add(item.Text);
			}
			cocoOptions.CocoExe = String.Join(";", (string[])tmp.ToArray(typeof(string)));
			
			ListView l = ((ListView)ControlDictionary["CocoListView"]);
			if (l.SelectedIndices.Count > 0)
				cocoOptions.Selected = l.SelectedIndices[0].ToString();
			else
				cocoOptions.Selected = "0";
			
			return true;
		}
		
		/// <summary>
		/// choose the location of Coco/R
		/// </summary>
		void SelectCocoClicked(object sender, EventArgs e){
			OpenFileDialog fdiag = new OpenFileDialog();
			fdiag.Title = "Choose the Coco/R executable you want to use !";
			fdiag.Filter = "Coco/R executable (*.exe)|*.exe|all files (*.*)|*.*";
			if (fdiag.ShowDialog() == DialogResult.OK) {
				((ListView)ControlDictionary["CocoListView"]).BeginUpdate();
				ListViewItem i = new ListViewItem(fdiag.FileName);
				i.Selected = true;
				((ListView)ControlDictionary["CocoListView"]).Items.Add(i);
				((ListView)ControlDictionary["CocoListView"]).EndUpdate();
				((Button)ControlDictionary["changeButton"]).Enabled = true;
				((Button)ControlDictionary["removeButton"]).Enabled = true;
			}
		}
		
		/// <summary>
		/// change the location of Coco/R
		/// </summary>
		void ChangeButtonClicked(object sender, EventArgs e){
			OpenFileDialog fdiag = new  OpenFileDialog();
			fdiag.Title = "Choose the Coco/R executable you want to use !";
			fdiag.Filter = "Coco/R executable (*.exe)|*.exe|all files (*.*)|*.*";
			fdiag.CheckPathExists = true;
			ListView l = (ListView)ControlDictionary["CocoListView"];
			if (l.SelectedItems != null)
				fdiag.InitialDirectory = l.SelectedItems[0].Text;
			if (fdiag.ShowDialog()== DialogResult.OK && l.SelectedItems != null) {
				l.BeginUpdate();
				l.SelectedItems[0].Text = fdiag.FileName;
				l.EndUpdate();
			}
		}
		
		/// <summary>
		/// remove a loaction of Coco/R
		/// </summary>
		void RemoveButtonClicked(object sender, EventArgs e){
			try {
				ListView l = (ListView)ControlDictionary["CocoListView"];
				l.Items.Remove(l.SelectedItems[0]);
				if (l.Items.Count == 0) {
					((Button)ControlDictionary["changeButton"]).Enabled = false;
					((Button)ControlDictionary["removeButton"]).Enabled = false;
				} else {
					l.Items[0].Selected = true;
				}
				
				
			} catch(Exception exc) {
				LoggingService.Error("Could not remove !" + exc );
			}
		}
		
		/// <summary>
		/// choose the location of the frame-Files
		/// </summary>
		void FrameButtonClicked(object sender, EventArgs e){
			FolderBrowserDialog fdiag = new FolderBrowserDialog();
			fdiag.Description = "Choose the location of the frame files !";
			if (fdiag.ShowDialog() == DialogResult.OK) {
				ControlDictionary["frameBox"].Text = fdiag.SelectedPath;
			}
		}
	}
}
