// <file>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Stefan Wallner" />
// </file>

 
 using System;
 using System.Diagnostics;
 using System.IO;
 using System.CodeDom.Compiler;
 using Microsoft.CSharp;
 
 using ICSharpCode.SharpDevelop.Internal;
 using ICSharpCode.Core;
 using ICSharpCode.SharpDevelop;	//IprojectService
 
 namespace CocoAddin.Tools
 {
 	/// <summary>
 	/// manages the Compilationprocess
 	/// </summary>
 	public class CocoCompilerManager
 	{
 		public CocoProperties properties;
 			
 		public CocoCompilerManager()
 		{
 			properties = new CocoProperties();
 		}
 			
 		/// <summary>
 		/// start the compiling
 		/// </summary>
 		public CSharpCompilerResult Compile(){
 			string output = "";
 			string error = "";
 			string exec = "";
 			
 			if (properties.InputFile == null)
 				throw new CocoAddinException("No valid copileable file found !");
 				
 			TempFileCollection tf = new TempFileCollection();
 			CSharpCompilerResult cr = null;
 			exec = properties.CocoParameterString;
 			
 			try {
 				
 				Executor.ExecWaitWithCapture(exec, tf, ref output, ref error);
 				cr = ParseOutput(tf, output);
 			
 			} catch (Exception e) {
 				LoggingService.Error("CocoCompilerManager: " + e);
 				return cr;
 			}
 			return cr;
 		}
 		
 		/// <summary>
 		/// parse the compile result
 		/// </summary>
 		CSharpCompilerResult ParseOutput(TempFileCollection tf, string file){
 			
 			string compilerOutput = "";
 			string next = "";
 			string last = "";
 			int errorNum = 0;
 			bool parserScanner = false;
 			StreamReader sr = new StreamReader(file, System.Text.Encoding.Default);
 			
 			if (sr==null) {
 				LoggingService.Error("Reading error !");
 				return null;
 			}
 			CompilerResults cr = new CompilerResults(tf);
 			
 			CompilerError error = null;	//an Error
 			
 			while (true){
 				last = next;
 				next = sr.ReadLine();
 				//at the end of the output
 				if (next == null){
 					int index;
 					int AllErrors;
 					
 					if (last.StartsWith("parser--")){ 	
 						LoggingService.Debug("parser error !"); 						 						
 						error = new CompilerError();
 						error.ErrorNumber = "0";
 						error.ErrorText = last;
 						error.FileName = properties.InputFile;
 						cr.Errors.Add(error);
 						errorNum++;
 						break;
 					} 	
 					
 					if (last.StartsWith("LL1 warning")){
 						LoggingService.Debug("LL1 warning"); 						 						
 						error = new CompilerError();
 						error.ErrorNumber = "3";
 						error.ErrorText = last;
 						error.FileName = properties.InputFile; 						
 						cr.Errors.Add(error);
 						errorNum++;
 					}
 					
 					if (last.LastIndexOf("deletable") > -1){
 						LoggingService.Debug("deletable");
 						error = new CompilerError();
 						error.ErrorNumber = "4";
 						error.ErrorText = last;
 						error.FileName = properties.InputFile;
 						cr.Errors.Add(error);
 						errorNum++;
 					}
 					
 					index = last.IndexOf("errors");
 					
 					if (index > 1) {
 						error = new CompilerError();
 						AllErrors = Int32.Parse(last.Substring(0, index));
 						Debug.WriteLine("Aufgetretene Fehler: " + AllErrors);
 						break;
 					}
 					break;
 					
 				} else {
 					int index, tmpIndex;
 					
 					compilerOutput += next + "\n";
 					if (next.StartsWith("-- line")){
 						errorNum++;
 						error = new CompilerError();
 						index = next.IndexOf("col")-1;
 						tmpIndex = next.IndexOf(":");
 						error.ErrorNumber = "2";
 						error.Line = Convert.ToInt32(next.Substring(8,index-8));
 						error.Column = Convert.ToInt32(next.Substring(index+5,tmpIndex-index-5));
 						error.FileName = properties.InputFile;	
 						error.ErrorNumber = errorNum.ToString();
 						error.ErrorText = next.Substring(tmpIndex+1,next.Length-tmpIndex-1);
 						
 						Debug.WriteLine("geparst:" + error);
 						cr.Errors.Add(error);
 					}
 					
 					if (next.StartsWith("parser + scanner generated")){
 						parserScanner = true;
 					}
 					continue;
 				}
 			}
 			sr.Close();
 			if(!parserScanner)	{ 
 				error = new CompilerError();
 				error.ErrorNumber = "2";
 				error.IsWarning = true;
 				error.FileName = properties.InputFile;
 				error.ErrorText = "no parser + scanner generated !";
 				cr.Errors.Add(error);
 			}
 			return new CSharpCompilerResult(cr, compilerOutput);
 		}
 	}
 }
