// <file>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Stefan Wallner" />
// </file>

 
 using System;
 using System.IO;
 using System.Windows.Forms;
 using System.Reflection;
 using System.Text;
 using System.Runtime.InteropServices;
 
 using ICSharpCode.Core;
 using ICSharpCode.SharpDevelop;
 using ICSharpCode.SharpDevelop.Internal.ExternalTool;
 
 namespace CocoAddin.Tools
 {
 	/// <summary>
	/// property managment
	/// </summary>
	public class CocoProperties
	{
		[DllImport("kernel32.dll", CharSet = CharSet.Auto)]
		static extern int GetShortPathName(
			[MarshalAs(UnmanagedType.LPTStr)]
			string path,
			[MarshalAs(UnmanagedType.LPTStr)]
			StringBuilder shortPath,
			int shortPathLength);
		
		private string pluginDir;
		
		//the ATG-File for Coco/R
		private string inputFile;
		
		public CocoProperties() {
			pluginDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
		}
		
		public string PluginDir {
			get { return pluginDir; } 
		}
		
		public string InputFile {
			get { return inputFile; }
			set { inputFile = value; }
		}
				
		public string CocoFileName {
			get { return PropertyService.Get("Addins.CocoAddin.CocoFileName",""); }
			set { PropertyService.Set("Addins.CocoAddin.CocoFileName", value); }
		}
			
		public string CocoPackage {
			get { return PropertyService.Get("Addins.CocoAddin.CocoPackage",""); }
			set { PropertyService.Set("Addins.CocoAddin.CocoPackage", value);}
		}
		
		public string CocoTrace {
			get { return TraceString; }
					
			set {	string frames = (string)value;
					OptionTrace = frames.IndexOf('A') > -1 						? true : false;
					OptionPrintFirstSetsFollowSets = frames.IndexOf('F') > -1 	? true : false;
					OptionPrintSyntaxGraph = frames.IndexOf('G') > -1 			? true : false;
					OptionTraceComputationFirstSets = frames.IndexOf('I') > -1 	? true : false;
					OptionListAnySyncSets = frames.IndexOf('J') > -1 			? true : false;
					OptionPrintStatistics = frames.IndexOf('P') > -1 			? true : false;
					OptionPrintSymbolTable = frames.IndexOf('S') > -1 			? true : false;
					OptionListCrossReferenceTable = frames.IndexOf('X') > -1 	? true : false;
			}
		}
		
		
		/// <summary>
		/// returns the place of Coco/R compiler generator, expects a compilergenerator in the directory of the addin
		/// </summary>
		public string CocoExe {
			get { 
					string back = PropertyService.Get("Addins.CocoAddin.CocoExe", pluginDir + Path.DirectorySeparatorChar + "Coco.exe");
					if (back.Length == 0) {
						back = pluginDir + Path.DirectorySeparatorChar + "Coco.exe";
						if (File.Exists(back)){
							PropertyService.Set("Addins.CocoAddin.CocoExe", back);
						} else {
							MessageService.ShowWarning("Coco/R-Addin didn't found the Coco/R compilergenerator at his expected location(" + 
							                            pluginDir+")!\n Please choose a location in menue Tools->Options->Tools->Coco/R.");
						}
					}
					return back;
			}
			set {
				PropertyService.Set("Addins.CocoAddin.CocoExe",value); 
			}
		}
		
		public string CocoFrame {
			get { return PropertyService.Get("Addins.CocoAddin.CocoFrame", ""); }
			set { PropertyService.Set("Addins.CocoAddin.CocoFrame", value); }
		}
		
 		/// <summary>
 		/// path for the Scanner.cs, Parser.cs and traceoutput
 		/// </summary>
		public string OutPutPath {
 			get { return Path.GetDirectoryName(InputFile);}
 		}
		
		public string Selected {
			get { return PropertyService.Get("Addins.CocoAddin.Selected",""); }
			set { PropertyService.Set("Addins.CocoAddin.CocoExe.Selected", value); }
		}
		
		public string OptionFilename {
			get { return PropertyService.Get("SharpDevelop.Addins.CocoAddin.CocoOptionFilename",""); }
			set { PropertyService.Set("SharpDevelop.Addins.CocoAddin.CocoOptionFilename", value); }
		}
		
		public string OptionPackage {
			get { return PropertyService.Get("SharpDevelop.Addins.CocoAddin.CocoOptionPackage",""); }
			set {
				PropertyService.Set("SharpDevelop.Addins.CocoAddin.CocoOptionPackage", value); 
			}
		}
		
		public string OptionExtended {
			get { return PropertyService.Get("SharpDevelop.Addins.CocoAddin.OptionExtended",""); }
			set { PropertyService.Set("SharpDevelop.Addins.CocoAddin.OptionExtended", value);}
		}
		
		public bool OptionTrace {
			get { return PropertyService.Get<bool>("SharpDevelop.Addins.CocoAddin.OptionTrace", true); }
			set { PropertyService.Set<bool>("SharpDevelop.Addins.CocoAddin.OptionTrace", value); }
		}
		
		public bool OptionPrintFirstSetsFollowSets {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionPrintFirstSetsFollowSets", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionPrintFirstSetsFollowSets", value); }
		}
		
		public bool OptionPrintSyntaxGraph {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionPrintFirstSetsFollowSets", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionPrintFirstSetsFollowSets", value); }
		}
		
		public bool OptionTraceComputationFirstSets {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionTraceComputationFirstSets", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionTraceComputationFirstSets", value); }
		}
		
		public bool OptionListAnySyncSets {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionListAnySyncSets", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionListAnySyncSets", value); }
		}
		
		public bool OptionPrintStatistics {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionPrintStatistics", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionPrintStatistics", value); }
		}
		
		public bool OptionPrintSymbolTable {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionPrintSymbolTable", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionPrintSymbolTable", value); }
		}
		
		public bool OptionListCrossReferenceTable {
			get { return PropertyService.Get<bool>("Addins.CocoAddin.OptionListCrossReferenceTable", true); }
			set { PropertyService.Set<bool>("Addins.CocoAddin.OptionListCrossReferenceTable", value); }
		}
		
		public ListView.ListViewItemCollection OptionCocoCompiler {
			
			get { return PropertyService.Get<ListView.ListViewItemCollection>("Addins.CocoAddin.CocoOptionCompiler", new ListView.ListViewItemCollection(new ListView())); }
			set { PropertyService.Set("Addins.CocoAddin.CocoOptionCompiler", value); }
		}
		
		public string TraceString {
			get {
				string back = "";
				if (OptionTrace) 						back += "A";
				if (OptionPrintFirstSetsFollowSets) 	back += "F";
				if (OptionPrintSyntaxGraph) 			back += "G";
				if (OptionTraceComputationFirstSets) 	back += "I";
				if (OptionListAnySyncSets) 				back += "J";
				if (OptionPrintStatistics) 				back += "P";
				if (OptionPrintSymbolTable) 			back += "S";
				if (OptionListCrossReferenceTable) 		back += "X";
				return back;
			}
		}
		
 		/// <summary>
 		/// returns the parameterstring for Coco/R using the parameters stored in the actual object
 		/// </summary>
		public string CocoParameterString {
			get {
				string coco = this.CocoExe;
				string inFile = this.InputFile;
				string frames = this.CocoFrame;
								
				try {
					PlatformID s = Environment.OSVersion.Platform;
					//if on windows with enabeld long directory names like "Eigene Dateien"
					//convert them to the old 8.3 Version using Windows API 
					if (s == PlatformID.Win32NT || s == PlatformID.WinCE || s == PlatformID.Win32S) {
						StringBuilder shortPath = new StringBuilder(200);
						int result = GetShortPathName(coco, shortPath, shortPath.Capacity);
						if (result != 0)	coco = shortPath.ToString();
						
						result = GetShortPathName(inFile, shortPath, shortPath.Capacity);
						if (result != 0)	inFile = shortPath.ToString();
						
						result = GetShortPathName(frames, shortPath, shortPath.Capacity);
						if (result != 0)	frames = shortPath.ToString();
					}		
				} catch(Exception ex) {
					LoggingService.Debug("Kein Windows ?" + ex);
				}
				
				string back = coco +" "+inFile;
				
				if (!CocoPackage.Equals("")) { back += " -namespace " + CocoPackage; }
				if (!frames.Equals("")) { back += " -frames " + frames; }
				if (!TraceString.Equals("")) { back += " -trace " + TraceString; }
				back += OptionExtended;
				return back;
			}
		}
	}
 }
